@@ -1,12 +1,7 @@
 /*
  * Java-Gnome Bindings Library
  *
- * Copyright 1998-2002 the Java-Gnome Team, all rights reserved.
- *
- * The Java-Gnome Team Members:
- *   Jean Van Wyk <jeanvanwyk@iname.com>
- *   Jeffrey S. Morgan <jeffrey.morgan@bristolwest.com>
- *   Dan Bornstein <danfuzz@milk.com>
+ * Copyright 1998-2004 the Java-Gnome Team, all rights reserved.
  *
  * The Java-Gnome bindings library is free software distributed under
  * the terms of the GNU Library General Public License version 2.
@@ -104,17 +99,19 @@ JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1notify_1add
 	jclass cls = (*env)->GetObjectClass(env, obj);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, namespace, NULL);
+	NotifyData *data;
+	guint ret;
 	if (methodID == NULL) {
 		methodID = (*env)->GetMethodID(env, cls, "notifyCallback", "(ILjava/lang/String;)V");
 		if (methodID == NULL)
-			return;
+			return -1;
 	}
-	NotifyData *data = g_new(NotifyData, 1);
+	data = g_new(NotifyData, 1);
 	data->env = env;
 	data->obj = obj;
 	data->methodID = methodID;
 	data->namespace = namespace;
-	guint ret = gconf_client_notify_add((GConfClient*)client, str, notifyCallback, data, notifyFree, &err);
+	ret = gconf_client_notify_add((GConfClient*)client, str, notifyCallback, data, notifyFree, &err);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
 	(*env)->ReleaseStringUTFChars(env, namespace, str);
 	return ret;
@@ -266,10 +263,12 @@ JNIEXPORT jintArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1all_1en
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
 	GSList* val = gconf_client_all_entries((GConfClient*)client, str, &err);
 	int i = 0;
+	jintArray ar;
+	guint len;
 	if (NULL == val)
 		return NULL;
-	guint len = g_slist_length(val);
-	jintArray ar = (*env)->NewIntArray(env, len);
+	len = g_slist_length(val);
+	ar = (*env)->NewIntArray(env, len);
 	for (i = 0; i < len; i++) {
 		GSList *item = g_slist_nth(val, i);
 		(*env)->SetIntArrayRegion(env, ar, i, 1, (jint*)item->data);
@@ -291,16 +290,19 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1all_
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
 	GSList* val = gconf_client_all_dirs((GConfClient*)client, str, &err);
 	int i = 0;
+	guint len;
+	jclass strCls;
+	jobjectArray ar;
 	if (NULL == val)
 		return NULL;
-	guint len = g_slist_length(val);
-	jclass strCls = (*env)->FindClass(env, "java/lang/String");
+	len = g_slist_length(val);
+	strCls = (*env)->FindClass(env, "java/lang/String");
 	if (NULL == strCls)
 		return NULL;
-	jobjectArray ar = (*env)->NewObjectArray(env, len, strCls, NULL);
+	ar = (*env)->NewObjectArray(env, len, strCls, NULL);
 	for (i = 0; i < len; i++) {
 		GSList *item = g_slist_nth(val, i);
-		jstring str = (*env)->NewStringUTF(env, (gchar*)item->data);
+	 	jstring str = (*env)->NewStringUTF(env, (gchar*)item->data);
 		(*env)->SetObjectArrayElement(env, ar, i, str);
 	}
 	(*env)->ReleaseStringUTFChars(env, dir, str);
@@ -432,10 +434,12 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_
 	GConfValueType vt = (GConfValueType)type;
 	GSList* val = gconf_client_get_list((GConfClient*)client, str, vt, &err);
 	int i = 0;
+	jclass aCls;
+	guint len;
+	jobjectArray ar;
 	if (NULL == val)
 		return NULL;
-	jclass aCls;
-	guint len = g_slist_length(val);
+	len = g_slist_length(val);
 	if (vt == GCONF_VALUE_STRING)
 		aCls = (*env)->FindClass(env, "java/lang/String");
 	else if (vt == GCONF_VALUE_INT)
@@ -448,7 +452,7 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_
 		aCls = (*env)->FindClass(env, "org/gnu/gconf/ConfSchema");
 	if (NULL == aCls)
 		return NULL;
-	jobjectArray ar = (*env)->NewObjectArray(env, len, aCls, NULL);
+	ar = (*env)->NewObjectArray(env, len, aCls, NULL);
 	for (i = 0; i < len; i++) {
 		GSList *item = g_slist_nth(val, i);
 		if (vt == GCONF_VALUE_STRING) {
@@ -456,27 +460,31 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_
 			(*env)->SetObjectArrayElement(env, ar, i, str);
 		} else if (vt == GCONF_VALUE_INT) {
 			jmethodID mid = (*env)->GetMethodID(env, aCls, "<init>", "(I)V");
+			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			jobject obj = (*env)->NewObject(env, aCls, mid, (jint)item->data);
+			obj = (*env)->NewObject(env, aCls, mid, (jint)item->data);
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
 		} else if (vt == GCONF_VALUE_FLOAT) {
 			jmethodID mid = (*env)->GetMethodID(env, aCls, "<init>", "(D)V");
+			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			jobject obj = (*env)->NewObject(env, aCls, mid, (jdouble*)item->data);
+			obj = (*env)->NewObject(env, aCls, mid, (jdouble*)item->data);
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
 		} else if (vt == GCONF_VALUE_BOOL) {
 			jmethodID mid = (*env)->GetMethodID(env, aCls, "<init>", "(Z)V");
+			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			jobject obj = (*env)->NewObject(env, aCls, mid, (jboolean)((gboolean)item->data));
+			obj = (*env)->NewObject(env, aCls, mid, (jboolean)((gboolean)item->data));
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
 		} else if (vt == GCONF_VALUE_SCHEMA) {
 			jmethodID mid = (*env)->GetMethodID(env, aCls, "<init>", "(I)V");
+			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			jobject obj = (*env)->NewObject(env, aCls, mid, (jint)item->data);
+			obj = (*env)->NewObject(env, aCls, mid, (jint)item->data);
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
 		}
 	}
@@ -591,15 +599,72 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1sch
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_set_list
- * Signature: (ILjava/lang/String;ILjava/util/List;[I)Z
+ * Signature: (ILjava/lang/String;I[Ljava/lang/Object;[I)Z
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1list
-  (JNIEnv *env, jclass cls, jint client, jstring key, jint type, jobject list, jintArray exception)
+  (JNIEnv *env, jclass cls, jint client, jstring key, jint type, jobjectArray list, jintArray exception)
 {
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gboolean ret = gconf_client_set_list((GConfClient*)client, str, (GConfValueType)type, 
-			(GSList*)setList(env, list), &err);
+	GConfValueType vt = (GConfValueType)type;
+	int i = 0;
+	jclass aCls = NULL;
+	jobject obj;
+	jfieldID fid;
+	jmethodID mid;
+	jsize len;
+	GSList* gslist;
+	gpointer data = NULL;
+	gboolean ret;
+	
+	len = (*env)->GetArrayLength(env, list);
+	gslist = g_slist_alloc();
+	for (i = 0; i < len; i++) {
+		obj = (*env)->GetObjectArrayElement(env, list, i);
+		if (vt == GCONF_VALUE_STRING) {
+			jstring value = (jstring)obj;
+			const gchar* v = (*env)->GetStringUTFChars(env, value, NULL);
+			data = (gpointer)v;
+		}
+		else if (vt == GCONF_VALUE_INT) {
+			jint value;
+			aCls = (*env)->FindClass(env, "java/lang/Integer");
+			mid = (*env)->GetMethodID(env, cls, "intValue", "()I");
+			if (NULL == mid)
+				return (jboolean)JNI_FALSE;
+			value = (*env)->CallIntMethod(env, obj, mid);
+			data = (gpointer)&value;
+		}
+		else if (vt == GCONF_VALUE_FLOAT) {
+			jdouble value;
+			aCls = (*env)->FindClass(env, "java/lang/Double");
+			mid = (*env)->GetMethodID(env, cls, "doubleValue", "()D");
+			if (NULL == mid)
+				return (jboolean)JNI_FALSE;
+			value = (*env)->CallDoubleMethod(env, obj, mid);
+			data = (gpointer)&value;
+		}
+		else if (vt == GCONF_VALUE_BOOL) {
+			jboolean value;
+			aCls = (*env)->FindClass(env, "java/lang/Boolean");
+			mid = (*env)->GetMethodID(env, cls, "booleanValue", "()Z");
+			if (NULL == mid)
+				return (jboolean)JNI_FALSE;
+			value = (*env)->CallBooleanMethod(env, obj, mid);
+			data = (gpointer)&value;
+		}
+		else if (vt == GCONF_VALUE_SCHEMA) {
+			jint value;
+			aCls = (*env)->FindClass(env, "org/gnu/gconf/ConfSchema");
+			fid = (*env)->GetFieldID(env, aCls, "handle", "I");
+			if (fid == NULL)
+				return (jboolean)JNI_FALSE;
+			value = (jint)(*env)->GetObjectField(env, obj, fid);
+			data = (gpointer)value;
+		}
+		gslist = g_slist_append(gslist, data);
+	}
+	ret = gconf_client_set_list((GConfClient*)client, str, vt, gslist, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
 	return (jboolean)ret;
